//
//	Includes
//
#include	"ColdFire.h"

//
//	Prototypes
//
void InitI2c(UINT32 clock);
__declspec(interrupt) void I2cHandler(void);
UINT32 I2cSend(UINT8 *data);
UINT32 I2cGet(UINT8 *data);
void SystemSleep(void);

//
//	Constants
//

//
//	Vars
//
UINT8	I2cData[4];		// Send & receive data buffer
UINT8	I2cIndex;			// Data store & read index
UINT8	I2cSize;			// Data transfer size

UINT8	I2cFinish;		// Data transfer finish flag
UINT8	I2cRecv;			// Address send mode flag

//
//	Init I2C controler
//
void InitI2c(UINT32 clock)
{
	MCF_GPIO_PASPAR = 0					// Set Port AS for I2C SCL & SDA purpose
		| MCF_GPIO_PASPAR_SCL_SCL	// Use PAS0 as a I2C SCL
		| MCF_GPIO_PASPAR_SDA_SDA;// Use PAS1 as a I2C SDA

	MCF_I2C_I2FDR = clock;			// I2C rate set
	MCF_I2C_I2ADR = 0x00;				// I2C slave adrs set (no use)

	MCF_I2C_I2CR = 0				// Set I2C control register
		| MCF_I2C_I2CR_IIEN		// Enable Interrupts
		| MCF_I2C_I2CR_IEN;		// Enable I2C module

	*(UINT32*)0x20000144 = (UINT32)I2cHandler;// Set interrupt vector (interrupt 17)

	MCF_INTC0_ICR17 = MCF_INTC_ICR_IL(1);			// Set interrupt level 1
	MCF_INTC0_IMRL &= ~MCF_INTC_IMRL_INT_MASK17;	// Reset interrupt controler mask bit
}

//
//	I2C interrupt handler
//
__declspec(interrupt) void I2cHandler(void)
{
	MCF_I2C_I2SR &= 0xfd;		// Cleat interrupt bit
	if(MCF_I2C_I2CR & MCF_I2C_I2CR_MSTA)	// Master mode bit set?
	{	// Master mode interrupt handler from here
		if(MCF_I2C_I2CR & MCF_I2C_I2CR_MTX)	// TX mode? (MTX bit = 1)
		{		// TX mode
			if(I2cFinish)		// Sent all data already?
				MCF_I2C_I2CR &= ~MCF_I2C_I2CR_MSTA;	// Clear MSTA bit(make end condition)
			else
			{	// Data still exists
				if(MCF_I2C_I2SR & MCF_I2C_I2SR_RXAK)// Check ACK from I2C slaves
				{	// No ACK
					MCF_I2C_I2CR &= ~MCF_I2C_I2CR_MSTA;	// Clear MSTA bit(end condition)
					I2cFinish = 2;	// Set error result code (NACK detected)
				}
				else// ACK has come from device
				{		// ACK has come
					if(I2cRecv)		// Recieve mode flag set then switch to recieve
					{	// Recieve mode flag set
						I2cRecv = 0;//Clear recieve mode flag first
						MCF_I2C_I2CR &= ~MCF_I2C_I2CR_MTX;	// Clear TX bit(go RX mode)
						(void)MCF_I2C_I2DR;	//Dummy read data register(will generate clock)
					}
					else
					{	// Continue transmit mode
						if(I2cSize == 0)		// Slave scan mode?
							MCF_I2C_I2CR &= ~MCF_I2C_I2CR_MSTA;	// Clear MSTA bit(make end condition)
						else
							MCF_I2C_I2DR = I2cData[I2cIndex++];// Send data
						if(I2cIndex == I2cSize)	// Data transfer end check
							I2cFinish = 1;	// Set normal result code
					}
				}
			}
		}
		else
		{	// RX mode (MTX bit = 0)
			if(I2cSize - I2cIndex == 2)				// last two data now?
				MCF_I2C_I2CR |= MCF_I2C_I2CR_TXAK;	// Set TXAK bit(to send NACK)
			else if(I2cSize - I2cIndex == 1)	// Final data now?
			{
				MCF_I2C_I2CR &= ~MCF_I2C_I2CR_MSTA;	// Clear MSTA bit(make end condition)
				I2cFinish = 1;	// Set normal result code
			}
			I2cData[I2cIndex] = MCF_I2C_I2DR;	// Save recieved data
			I2cIndex++;				// Advance index
		}
	}
}

//
//	Send I2C data
//
UINT32 I2cSend(UINT8 *data)
{
int	i;
	I2cFinish = 0;	// Clear finish bit 
	I2cIndex = 0;		// Clear send index
  I2cSize = data[0]; // Set data send size
	for(i = 0; i < I2cSize; i++)// Make copy of send data
		I2cData[i] = data[i + 2];	// Save data to be sent
	I2cRecv = 0;		// Clear recieve mode flag
	MCF_I2C_I2CR |= (MCF_I2C_I2CR_MTX | MCF_I2C_I2CR_MSTA);// Set MSTA & MTX bit same time
  MCF_I2C_I2DR = data[1] * 2;	// Send adrs (RW bit = 0)
	while(!I2cFinish || MCF_I2C_I2SR & MCF_I2C_I2SR_IBB)SystemSleep();// Wait end of session
	return (I2cFinish == 1);		// Check error happen
}

//
//	Get I2C data
//
UINT32 I2cGet(UINT8 *data)
{
int i;
	I2cFinish = 0;			// Clear finish bit 
	I2cIndex = 0;				// Clear send index
  I2cSize = data[0];	// Set data send size
	I2cRecv = 1;				// Set recieve mode flag
	MCF_I2C_I2CR |= (MCF_I2C_I2CR_MTX | MCF_I2C_I2CR_MSTA);// Set MSTA & MTX bit same time
	if(I2cSize == 1)		// Send data size = 1?
		MCF_I2C_I2CR |= MCF_I2C_I2CR_TXAK;	// Set TXAK bit from begining
	else
		MCF_I2C_I2CR &= ~MCF_I2C_I2CR_TXAK;	// Clear TXAK bit first
  MCF_I2C_I2DR = data[1] * 2 + 1;	// Send adrs (RW bit = 1)
	while(!I2cFinish || MCF_I2C_I2SR & MCF_I2C_I2SR_IBB)SystemSleep();// Wait session finish
	for(i = 0; i < I2cSize; i++)	// Copy recieved data to user pointer
		data[i + 2] = I2cData[i];		// Set read data
	return (I2cFinish == 1);			// Check error happen
}

//
//	SystemSleep system call
//
asm void SystemSleep(void)
{
  move 0x210,a0		// Get SystemSleep internal address to a0
  jmp (a0)				// Jump SystemSleep library
}
